/*****************************************************
Chip type               : ATtiny43U
AVR Core Clock frequency: 8,000000 MHz
*****************************************************/

#include <tiny43u.h>
#include <delay.h>
#define CPU_FREQ 8000000

#define V_bat_min (((int)11*255)/18)

#define out_1 PORTB.4
#define out_2 PORTB.5
#define ocp_led PORTB.7

unsigned char t_imp, t_pause, n_imp, v_bat;
unsigned char tmr [7];
unsigned char ADC_0, ADC_1, ADC_2, ADC_3; 


void ports_init ()
{
DDRA=0b01010000; PORTA=0b10000000;

DDRB=0b11111111; PORTB=0b00000000;
}

void timer_0_init ()
{
TCCR0A |= 1<<WGM01; //ctc
TCCR0B |= 1<<CS02|1<<CS00; //div by 1024
OCR0A = 0x9C; //156  - 50Hz
TIMSK0 |= 1<<OCIE0A; // int on compare match
}

void timer_1_init ()
{
//TCCR1A |= 1<<WGM11|1<<WGM10; //fast PWM, outputs disconnected
//TCCR1B |= 1<<WGM12|1<<CS11 ;   //top OCR1A, div by 8
}

void acmp_init ()
{
ACSR |= 1<<ACBG;
}

void adc_init ()
{
ADCSRA |= 1<<ADEN|1<<ADPS2|1<<ADPS1|1<<ADPS0;
ADCSRB |= 1<<ADLAR; 
ADCSRA |= 1<< ADSC; 
}

interrupt [TIM0_COMPA] void timer0_comp (void)
{
    char i; 
    for (i=0; i<7; i++) {if (tmr [i]) tmr [i]--;}
    
    #asm("wdr")
}

void ADC_read ()
{
static char ADC_select = 1;
static char counter = 64;
static unsigned int v_bat_tmp;

if (!ADCSRA.ADSC) 
    {
    switch (ADC_select)
    {
    case 0: ADMUX = 0; ADC_select = 1; ADC_3 = ADCH; break;
    case 1: ADMUX = 1; ADC_select = 2; ADC_0 = ADCH; break;
    case 2: ADMUX = 2; ADC_select = 3; ADC_1 = ADCH; break;
    case 3: ADMUX = 3; ADC_select = 0; ADC_2 = ADCH; break;
    }
    
     if (counter) {v_bat_tmp += ADC_3; counter--;} 
     else {v_bat = v_bat_tmp >>6; v_bat_tmp = 0; counter = 64;}
     ADCSRA |= 1<< ADSC;
    }
}

void convert_ADC ()
{
 t_imp = ((unsigned int)ADC_0 * 2*25)>>8;
 t_pause = ((unsigned int)ADC_1 * 2*25)>>8;
 n_imp = ((unsigned int)ADC_2 * 10)>>8;
}

void blinking_led ()
{ 
 ocp_led = ~ocp_led; delay_ms (400) ; 
}

void battery_check ()
{
while (v_bat < V_bat_min) {out_1 = 0; out_2 = 0; blinking_led (); ADC_read (); convert_ADC ();}
ocp_led = 0;
}

enum state {all_off, out1_on, out1_off, out2_on, out2_off, all_on} out_state;

void out2performing ()
{
    static char n_imp_tmp;   
   
   if (out_state == all_off) {out_state = out1_on; tmr[1] = t_imp; tmr[0] = 5; n_imp_tmp = n_imp;}
   if ((out_state == out1_on)&&(! tmr[1])) {out_state = out1_off; tmr[2] = t_pause;}
   if ((out_state == out1_off)&&(! tmr[2])&&(n_imp_tmp)) {out_state = out1_on; tmr[1] = t_imp; tmr[0] = 5; n_imp_tmp-- ;}  
   else if ((!n_imp_tmp)&&(out_state == out1_off)) {out_state = out2_on; n_imp_tmp = n_imp; tmr[0] = 5; tmr[1] = t_imp;}
   if ((out_state == out2_on)&&(! tmr[1])) {out_state = out2_off; tmr[2] = t_pause;}
   if ((out_state == out2_off)&&(! tmr[2])&&(n_imp_tmp)) {out_state = out2_on; tmr[1] = t_imp; tmr[0] = 5; n_imp_tmp-- ;}
   else if((!n_imp_tmp)&&(out_state == out2_off)) {out_state = out1_on; n_imp_tmp = n_imp; tmr[0] = 5; tmr[1] = t_imp;}
   
   
  switch (out_state)
  {
   case all_off: out_1 = 0; out_2 = 0; break;
   case out1_on: out_1 = 1; out_2 = 0; break;
   case out1_off: out_1 = 0; out_2 = 0; break;
   case out2_on: out_1 = 0; out_2 = 1; break;
   case out2_off: out_1 = 0; out_2 = 0; break;
   case all_on: out_1 = 1; out_2 = 1; break;
   default: break;
  }
 }

void ocp ()
{
while (((out_state == out1_on)||(out_state == out2_on))&&(ACSR.ACO)&&(!tmr[0])) {out_1 = 0; out_2 = 0; ocp_led = 1; tmr[6] = 255; out_state = all_off;};
while (tmr[6]);
ocp_led = 0;
}


void main(void)
{
#pragma optsize-
CLKPR |= 1<<CLKPCE;
CLKPR=0x00;    // Crystal Oscillator division factor: 1
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

ports_init ();
acmp_init ();
adc_init ();
timer_0_init ();
timer_1_init ();

#pragma optsize-
#asm("wdr")
WDTCSR=0x18;
WDTCSR |= 1<<WDE|1<<WDP2; // 0,25s 
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

#asm("sei")

out_state = all_off;
t_imp=25, t_pause=25, n_imp=5, v_bat=200;

while (1)
      {
       ADC_read ();
       convert_ADC ();
       battery_check ();
       out2performing ();
       ocp (); 
       }
}
